local Talented = Talented
local L = LibStub("AceLocale-3.0"):GetLocale("Talented")

function Talented:ApplyCurrentTemplate()
	local template = self.template
	local is_pet = not RAID_CLASS_COLORS[template.class]
	if is_pet then
		if not self.GetPetClass or self:GetPetClass() ~= template.class then
			self:Print(L["Sorry, I can't apply this template because it doesn't match your pet's class!"])
			self.mode = "view"
			self:UpdateView()
			return
		end
	elseif select(2, UnitClass"player") ~= template.class then
		self:Print(L["Sorry, I can't apply this template because it doesn't match your class!"])
		self.mode = "view"
		self:UpdateView()
		return
	end
	local count = 0
	local current = is_pet and self.pet_current or self:GetActiveTalents()
	-- check if enough talent points are available
	local available = GetUnspentTalentPoints(nil, is_pet)
	for tab, tree in ipairs(self:UncompressSpellData(template.class)) do
		for index = 1, #tree do
			local delta = template[tab][index] - current[tab][index]
			if delta > 0 then
				count = count + delta
			end
		end
	end
	if count == 0 then
		self:Print(L["Nothing to do"])
		self.mode = "view"
		self:UpdateView()
	elseif count > available then
		self:Print(L["Sorry, I can't apply this template because you don't have enough talent points available (need %d)!"], count)
		self.mode = "view"
		self:UpdateView()
	else
		self:EnableUI(false)
		self:ApplyTalentPoints()
	end
end

function Talented:ApplyTalentPoints()
	assert (GetCVarBool"previewTalents")

	local template = self.template
	local pet = not RAID_CLASS_COLORS[template.class]

	ResetGroupPreviewTalentPoints(pet)

	local cp = GetUnspentTalentPoints(nil, pet)
	local group = GetActiveTalentGroup()

	while true do
		local missing, set
		for tab, tree in ipairs(self:UncompressSpellData(template.class)) do
			local ttab = template[tab]
			for index = 1, #tree do
				local rank = select(9, GetTalentInfo(tab, index, nil, pet))

				local delta = ttab[index] - rank
				if delta > 0 then
					AddPreviewTalentPoints(tab, index, delta, pet, group)
					cp = cp - delta
					local nrank = select(9, GetTalentInfo(tab, index, nil, pet))
					if nrank < ttab[index] then
						missing = true
					elseif nrank > rank then
						set = true
					end
				end
			end
		end
		if not missing then break end
		assert(set) -- make sure we did something
	end
	if cp < 0 then
		Talented:Print(L["Error while applying talents! Not enough talent points!"])
		ResetGroupPreviewTalentPoints(pet)
		Talented:EnableUI(true)
	else
		LearnPreviewTalents(pet)
	end
end

function Talented:CheckTalentPointsApplied()
	local template = self.template
	local pet = not RAID_CLASS_COLORS[template.class]
	local failed
	for tab, tree in ipairs(self:UncompressSpellData(template.class)) do
		local ttab = template[tab]
		for index = 1, #tree do
			local delta = ttab[index] - select(5, GetTalentInfo(tab, index, nil, pet))
			if delta > 0 then
				failed = true
				break
			end
		end
	end
	if failed then
		Talented:Print(L["Error while applying talents! some of the request talents were not set!"])
	else
		local cp = GetUnspentTalentPoints(nil, pet)
		Talented:Print(L["Template applied successfully, %d talent points remaining."], cp)
	end
	Talented:OpenTemplate(self.pet_current)
	Talented:EnableUI(true)
end
