local Talented = Talented
local L = LibStub("AceLocale-3.0"):GetLocale("Talented")

local WH_MAP = "0zMcmVokRsaqbdrfwihuGINALpTjnyxtgevE"
local WH_PET_INFO_CLASS = {
	Ferocity = {1,2,7,11,12,25,37,39,44,45,46},
	Tenacity = {4,5,6,8,9,20,21,32,42,43},
	Cunning = {3,24,26,27,30,31,33,34,35,38,41},
}
-- local WH_PET_INFO_ALT = {7,24,26,27,30,34,37,38}

local TALENTED_MAP = "012345abcdefABCDEFmnopqrMNOPQRtuvwxy*"
local TALENTED_CLASS_CODE = {
	Ferocity = "t",
	Cunning = "w",
	Tenacity = "*",
	["t"] = "Ferocity",
	["w"] = "Cunning",
	["*"] = "Tenacity",
}

local function GetPetClassByFamily(index)
	for class, v in pairs(WH_PET_INFO_CLASS) do
		for i = 1, #v do
			if v[i] == index then return class end
		end
	end
end

local function GetPetFamilyForClass(class)
	return WH_PET_INFO_CLASS[class][1]
end

local function map(code, src, dst)
	local temp = {}
	for i = 1, string.len(code) do
		local index = assert(src:find(code:sub(i, i), nil, true))
		temp[i] = dst:sub(index, index)
	end
	return table.concat(temp)
end

local function ImportCode(code)
	local a = (WH_MAP:find(code:sub(1, 1), nil, true) - 1) * 10
	local b = (WH_MAP:find(code:sub(2, 2), nil, true) - 1) / 2
	if b ~= math.floor(b) then return end
	local family = a + b
	local class = GetPetClassByFamily(family)

	return TALENTED_CLASS_CODE[class]..map(code:sub(3), WH_MAP, TALENTED_MAP)
end

local function ExportCode(code)
	local class = TALENTED_CLASS_CODE[code:sub(1, 1)]
	local family = GetPetFamilyForClass(class)

	local a = math.floor(family / 10)
	local b = (family - (a * 10)) * 2 + 1
	return WH_MAP:sub(a + 1, a + 1)..WH_MAP:sub(b, b)..map(code:sub(2), TALENTED_MAP, WH_MAP)
end

local function FixImportTemplate(self, template)
	local data = self:UncompressSpellData(template.class)[1]
	template = template[1]
	for index, info in ipairs(data) do
		if info.inactive then
			if index > 1 and info.row == data[index - 1].row and info.column == data[index - 1].column then
				template[index - 1] = template[index] + template[index - 1]
			elseif index < #data and info.row == data[index + 1].row and info.column == data[index + 1].column then
				template[index + 1] = template[index] + template[index + 1]
			end
		end
	end
end

local function FixExportTemplate(self, template)
	local data = self:UncompressSpellData(template.class)[1]
	template = template[1]
	for index, info in ipairs(data) do
		if info.inactive then
			if index > 1 and info.row == data[index - 1].row and info.column == data[index - 1].column then
				template[index - 1] = template[index] + template[index - 1]
			end
		end
	end
end

function Talented:ImportWhpetTemplate(url, dst)
	local s, _, code = url:find(".*/%?petcalc=(.*)$")
	if not s or not code then return end
	code = ImportCode(code)
	if not code then return end
	local res, val, class = pcall(self.StringToTemplate, self, code, dst)
	if not res then
		return
	end
	dst.class = class
	FixImportTemplate(self, dst)
	return dst
end

function Talented:ExportWhpetTemplate(template)
	if RAID_CLASS_COLORS[template.class] then return end
	FixExportTemplate(self, template)
	local code = ExportCode(self:TemplateToString(template))
	FixImportTemplate(self, template)
	if code then
		return L["http://www.wowhead.com/?petcalc=%s"]:format(code)
	end
end


Talented.importers["/%?petcalc="] = Talented.ImportWhpetTemplate
